@@ -8,7 +8,7 @@ extract_msg.exceptions
 This module contains the set of extract_msg exceptions.
 """
 
-# add logger bus
+# Add logger bus.
 logger = logging.getLogger(__name__)
 logger.addHandler(logging.NullHandler())
 
@@ -65,6 +65,12 @@ class UnknownTypeError(Exception):
     """
     pass
 
+class UnsupportedMSGTypeError(NotImplementedError):
+    """
+    An exception that is raised when an MSG class is recognized by not
+    supported.
+    """
+
 class UnrecognizedMSGTypeError(TypeError):
     """
     An exception that is raised when the module cannot determine how to properly
