@@ -457,7 +457,7 @@ def knownMsgClass(classType : str) -> bool:
         return True
 
     for item in constants.KNOWN_CLASS_TYPES:
-        if classType.startsWith(item):
+        if classType.startswith(item):
             return True
 
     return False
